import asyncio
from abc import ABC
from http import HTTPStatus
from typing import Any, Dict, Mapping, Optional

import aiohttp
import fast_json
from aiohttp import hdrs
from yarl import URL


TResponse = Dict[str, Any]


class BaseClient(ABC):
    __slots__ = ("_url", "_session")

    def __init__(
        self,
        url: URL,
        auth_token: str,
        connector: Optional[aiohttp.TCPConnector] = None,
        loop: Optional[asyncio.AbstractEventLoop] = None,
        default_timeout: Optional[aiohttp.ClientTimeout] = None,
        **kwargs: Any,
    ):
        loop = loop or asyncio.get_event_loop()
        self._url = url
        self._session = aiohttp.ClientSession(
            connector=connector,
            json_serialize=fast_json.dumps,
            loop=loop,
            timeout=default_timeout or aiohttp.client.DEFAULT_TIMEOUT,
            headers={
                hdrs.ACCEPT: "application/json",
                hdrs.ACCEPT_ENCODING: "gzip, deflate",
                hdrs.CONNECTION: "keepalive",
                hdrs.AUTHORIZATION: auth_token,
            },
            **kwargs,
        )

    @property
    def url(self) -> URL:
        return self._url

    async def close(self) -> None:
        await self._session.close()

    async def _request(
        self,
        method: str,
        path: str,
        params: Optional[Mapping[str, str]] = None,
        data: Optional[Any] = None,
        json: Optional[Any] = None,
        **kwargs: Any,
    ) -> Optional[TResponse]:
        url = self._url / path.strip("/")
        async with self._session.request(
            method,
            url,
            params=params,
            data=data,
            json=json,
            **kwargs,
        ) as response:
            response.raise_for_status()
            content_type = response.headers.get(hdrs.CONTENT_TYPE)
            if (response.status == HTTPStatus.OK) and (
                content_type == "application/json"
            ):
                return await response.json(loads=fast_json.loads)
            return None

    async def _get(
        self,
        path: str,
        *,
        params: Optional[Mapping[str, str]] = None,
        **kwargs: Any,
    ) -> Optional[TResponse]:
        return await self._request(hdrs.METH_GET, path, params=params, **kwargs)

    async def _post(
        self,
        path: str,
        *,
        data: Optional[Any] = None,
        json: Optional[Any] = None,
        **kwargs: Any,
    ) -> Optional[TResponse]:
        return await self._request(
            hdrs.METH_POST,
            path,
            data=data,
            json=json,
            **kwargs,
        )



from .clients import BridgeClient


class JambotClient:
    def __init__(self, bridge: BridgeClient):
        self._bridge = bridge

    @property
    def bridge(self) -> BridgeClient:
        return self._bridge


__all__ = ("JambotClient",)



from datetime import datetime
from typing import Any, Mapping
from uuid import UUID

from jambot_client.clients.base_client import BaseClient


class BridgeClient(BaseClient):
    async def send_event(
        self,
        event_type: str,
        payload: Mapping[str, Any],
        bot_id: UUID,
        user_id: UUID,
        platform: str,
    ) -> None:
        await self._post(
            path="api/v1/events",
            json={
                "event": {
                    "event_type": event_type,
                    "payload": payload,
                    "event_ts": int(datetime.utcnow().timestamp()),
                },
                "bot": {
                    "bot_id": bot_id,
                },
                "user": {
                    "user_id": user_id,
                    "platform": platform,
                },
            },
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jambot_client', 'jambot_client.clients', 'jambot_client.clients.bridge']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0',
 'fast-json>=0.3.2,<0.4.0',
 'pydantic>=1.9.0,<2.0.0',
 'yarl>=1.7.2,<2.0.0']

setup_kwargs = {
    'name': 'jambot-client',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'Vladislav Bakaev',
    'author_email': 'vlad@bakaev.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
